"""Hedging helpers (`fuggers_py.analytics.risk.hedging`).

The exported helpers cover duration and DV01 hedge ratios plus simple
portfolio aggregation utilities.
"""

from __future__ import annotations

from .hedge_ratio import HedgeDirection, HedgeRecommendation, duration_hedge_ratio, dv01_hedge_ratio
from .portfolio import PortfolioRisk, Position, aggregate_portfolio_risk

__all__ = [
    "duration_hedge_ratio",
    "dv01_hedge_ratio",
    "aggregate_portfolio_risk",
    "HedgeDirection",
    "HedgeRecommendation",
    "PortfolioRisk",
    "Position",
]
